/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2022 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.api.networking.v1;

import net.fabricmc.fabric.api.event.Event;
import net.fabricmc.fabric.impl.base.event.QuiltCompatEvent;
import net.minecraft.class_1297;
import net.minecraft.class_3222;

/**
 * Events related to a tracking entities within a player's view distance.
 *
 * @deprecated Use Quilt Networking's {@link org.quiltmc.qsl.networking.api.EntityTrackingEvents} instead.
 */
@Deprecated
public final class EntityTrackingEvents {
	/**
	 * An event that is called before player starts tracking an entity.
	 * Typically, this occurs when an entity enters a client's view distance.
	 * This event is called before the player's client is sent the entity's {@link class_1297#method_18002() spawn packet}.
	 */
	public static final Event<StartTracking> START_TRACKING = QuiltCompatEvent.fromQuilt(org.quiltmc.qsl.networking.api.EntityTrackingEvents.BEFORE_START_TRACKING,
			startTracking -> startTracking::onStartTracking,
			invokerGetter -> (trackedEntity, player) -> invokerGetter.get().beforeStartTracking(trackedEntity, player)
	);

	/**
	 * An event that is called after a player has stopped tracking an entity.
	 * The client at this point was sent a packet to {@link net.minecraft.class_2716 destroy} the entity on the client.
	 * The entity still exists on the server.
	 */
	public static final Event<StopTracking> STOP_TRACKING = QuiltCompatEvent.fromQuilt(org.quiltmc.qsl.networking.api.EntityTrackingEvents.STOP_TRACKING,
			stopTracking -> stopTracking::onStopTracking,
			invokerGetter -> (trackedEntity, player) -> invokerGetter.get().onStopTracking(trackedEntity, player)
	);

	@FunctionalInterface
	public interface StartTracking {
		/**
		 * Called before an entity starts getting tracked by a player.
		 *
		 * @param trackedEntity the entity that will be tracked
		 * @param player the player that will track the entity
		 */
		void onStartTracking(class_1297 trackedEntity, class_3222 player);
	}

	@FunctionalInterface
	public interface StopTracking {
		/**
		 * Called after an entity stops getting tracked by a player.
		 *
		 * @param trackedEntity the entity that is no longer being tracked
		 * @param player the player that is no longer tracking the entity
		 */
		void onStopTracking(class_1297 trackedEntity, class_3222 player);
	}

	private EntityTrackingEvents() {
	}
}
