/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2022 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.impl.networking;

import io.netty.util.concurrent.Future;
import io.netty.util.concurrent.GenericFutureListener;
import org.jetbrains.annotations.Nullable;
import net.fabricmc.fabric.api.networking.v1.FabricPacket;
import net.fabricmc.fabric.api.networking.v1.PacketSender;
import net.minecraft.class_2540;
import net.minecraft.class_2596;
import net.minecraft.class_2960;
import net.minecraft.class_7648;

final class Quilt2FabricPacketSender implements PacketSender {
	final org.quiltmc.qsl.networking.api.PacketSender<?> sender;

	protected Quilt2FabricPacketSender(org.quiltmc.qsl.networking.api.PacketSender<?> sender) {
		this.sender = sender;
	}

	@Override
	public class_2596<?> createPacket(class_2960 channelName, class_2540 buf) {
		return this.sender.createPacket(channelName, buf);
	}

	@Override
	public class_2596<?> createPacket(FabricPacket packet) {
		return QuiltUtils.createS2CPacket(packet, this::createPacket);
	}

	@Override
	public void sendPacket(class_2596<?> packet) {
		this.sender.sendPacket(packet);
	}

	@Override
	public void sendPacket(class_2596<?> packet, @Nullable GenericFutureListener<? extends Future<? super Void>> callback) {
		this.sender.sendPacket(packet, GenericFutureListenerHolder.create(callback));
	}

	@Override
	public void sendPacket(class_2596<?> packet, @Nullable class_7648 callback) {
		this.sender.sendPacket(packet, callback);
	}

	@Override
	public void sendPacket(class_2960 channel, class_2540 buf, @Nullable GenericFutureListener<? extends Future<? super Void>> callback) {
		this.sender.sendPacket(channel, buf, GenericFutureListenerHolder.create(callback));
	}
}
